/*
 * Ubuntu One Files - access Ubuntu One cloud storage on Android platform.
 * 
 * Copyright (C) 2011 Canonical Ltd.
 * Author: Michał Karnicki <michal.karnicki@canonical.com>
 *   
 * This file is part of Ubuntu One Files.
 *  
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *  
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.files.activity;

import greendroid.app.GDActivity;
import greendroid.widget.ActionBar;
import greendroid.widget.LoaderActionBarItem;

import java.util.ArrayList;
import java.util.HashMap;

import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.view.View;
import android.widget.ListView;
import android.widget.Toast;

import com.ubuntuone.android.files.R;
import com.ubuntuone.android.files.util.ServicesAdapter;
import com.ubuntuone.android.files.util.ServicesStatus;
import com.ubuntuone.android.files.util.UIUtil;
import com.ubuntuone.android.files.util.ServicesStatus.ServiceNames;
import com.ubuntuone.android.files.util.ServicesStatus.ServiceTags;
import com.ubuntuone.android.files.Analytics;

import com.google.android.apps.analytics.GoogleAnalyticsTracker;

public class StatusActivity extends GDActivity {
	private static final String TAG = StatusActivity.class.getSimpleName();
	
	private Handler mHandler;
	
	private GoogleAnalyticsTracker mTracker;

	private LoaderActionBarItem mLoaderItem;
	
	private ListView listView;
	
	private ServicesAdapter listAdapter;
	
	private ArrayList<ServicesStatus.ServiceStatusRecord> services;
	
	private UpdateStatusTask updateStatusTask;
	
	String exceptionMsg = null;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		mTracker = GoogleAnalyticsTracker.getInstance();
		mTracker.start(Analytics.U1F_ACCOUNT, this);
		mTracker.trackPageView(TAG);

		setActionBarContentView(R.layout.activity_status);
		mLoaderItem = (LoaderActionBarItem) addActionBarItem(getActionBar()
				.newActionBarItem(LoaderActionBarItem.class)
				.setDrawable(R.drawable.gd_action_bar_refresh),
				ActionBar.NONE);
		
		mHandler = new Handler();
		
		services = new ArrayList<ServicesStatus.ServiceStatusRecord>();
		listView = (ListView)findViewById(R.id.service_list);
		listAdapter = new ServicesAdapter(this, R.layout.list_status_row, services);
		listView.setAdapter(listAdapter);
	}
	
	@Override
	protected void onResume() {
		super.onResume();
		maybeUpdateStatusTask();
	}
	
	public void onRefreshClick(View v) {
		maybeUpdateStatusTask();
	}
	
	private void maybeUpdateStatusTask() {
		if (updateStatusTask == null || 
				AsyncTask.Status.FINISHED.equals(updateStatusTask.getStatus())) {
			updateStatusTask = new UpdateStatusTask();
			updateStatusTask.execute();
		}
	}

	private class UpdateStatusTask extends AsyncTask<Void, Void, HashMap<String, Integer> > {
		
		@Override
		protected void onPreExecute() {
			super.onPreExecute();
			UIUtil.showToast(StatusActivity.this, R.string.refreshing);
			showProgress();
		}
		
		@Override
		protected HashMap<String, Integer> doInBackground(
				Void... params) {
			HashMap<String, Integer> status = null;
			try {
				 status = ServicesStatus.getServicesStatus();
			} catch (Exception e) {
				exceptionMsg = e.getLocalizedMessage();
				e.printStackTrace();
			}
			return status;
		}

		@Override
		protected void onPostExecute(HashMap<String, Integer> result) {
			super.onPostExecute(result);
			hideProgress();
			if (exceptionMsg != null) {
				Toast.makeText(StatusActivity.this, exceptionMsg, 
						Toast.LENGTH_SHORT).show();
				exceptionMsg = null;
			}
			if (result == null)
				return;
			
			// update status icons
			Integer status = null;
			
			services.clear();
			
			if ((status = result.get(ServiceTags.SERVICE_GENERAL)) != null) {
				services.add(new ServicesStatus.ServiceStatusRecord(
						ServiceNames.SERVICE_GENERAL, status));
			}
			
			if ((status = result.get(ServiceTags.SERVICE_FILES)) != null) {
				services.add(new ServicesStatus.ServiceStatusRecord(
						ServiceNames.SERVICE_FILES, status));
			}
			
			if ((status = result.get(ServiceTags.SERVICE_CONTACTS)) != null) {
				services.add(new ServicesStatus.ServiceStatusRecord(
						ServiceNames.SERVICE_CONTACTS, status));
			}
			
			if ((status = result.get(ServiceTags.SERVICE_MOBILE_CONTACTS)) != null) {
				services.add(new ServicesStatus.ServiceStatusRecord(
						ServiceNames.SERVICE_MOBILE_CONTACTS, status));
			}
			
			if ((status = result.get(ServiceTags.SERVICE_MOBILE_STREAMING)) != null) {
				services.add(new ServicesStatus.ServiceStatusRecord(
						ServiceNames.SERVICE_MOBILE_STREAMING, status));
			}
			
			if ((status = result.get(ServiceTags.SERVICE_MUSIC_STORE)) != null) {
				services.add(new ServicesStatus.ServiceStatusRecord(
						ServiceNames.SERVICE_MUSIC_STORE, status));
			}
			
			if ((status = result.get(ServiceTags.SERVICE_NOTES)) != null) {
				services.add(new ServicesStatus.ServiceStatusRecord(
						ServiceNames.SERVICE_NOTES, status));
			}
			
			if ((status = result.get(ServiceTags.SERVICE_BOOKMARKS)) != null) {
				services.add(new ServicesStatus.ServiceStatusRecord(
						ServiceNames.SERVICE_BOOKMARKS, status));
			}
			
			// separator
			services.add(new ServicesStatus.ServiceStatusRecord(
					getString(R.string.status_description), 0));
			
			services.add(new ServicesStatus.ServiceStatusRecord(
					getString(R.string.service_ok), 
					-ServicesStatus.StatusCodes.STATUS_OK));
			services.add(new ServicesStatus.ServiceStatusRecord(
					getString(R.string.service_disruption),
					-ServicesStatus.StatusCodes.STATUS_DISRUPTION));
			services.add(new ServicesStatus.ServiceStatusRecord(
					getString(R.string.service_outage),
					-ServicesStatus.StatusCodes.STATUS_OUTAGE));
			services.add(new ServicesStatus.ServiceStatusRecord(
					getString(R.string.service_info),
					-ServicesStatus.StatusCodes.STATUS_INFO));
			
			listAdapter.notifyDataSetChanged();
		}
		
	};
	
	private void showProgress() {
		mHandler.post(new Runnable() {
			public void run() {
				if (mLoaderItem != null)
					mLoaderItem.setLoading(true);
			}
		});
	}
	
	private void hideProgress() {
		mHandler.post(new Runnable() {
			public void run() {
				if (mLoaderItem != null)
					mLoaderItem.setLoading(false);
			}
		});
	}

	@Override
	public void onDestroy() {
		mTracker.dispatch();
		mTracker.stop();
		super.onDestroy();
	}
}
