package com.ubuntuone.android.files.fragment;

import greendroid.widget.AsyncImageView;
import greendroid.widget.AsyncImageView.OnImageViewLoadListener;

import java.lang.ref.WeakReference;
import java.util.ArrayList;

import android.app.Activity;
import android.graphics.Bitmap;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.text.Html;
import android.text.method.LinkMovementMethod;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import com.ubuntu.sso.UbuntuSingleSignOnAPI;
import com.ubuntu.sso.entry.CaptchaResponse;
import com.ubuntu.sso.entry.ServerResponse;
import com.ubuntu.sso.entry.ServerResponse.Error;
import com.ubuntuone.android.files.Constants;
import com.ubuntuone.android.files.R;
import com.ubuntuone.android.files.util.HttpManager;
import com.ubuntuone.android.files.util.RegisterUserTask;
import com.ubuntuone.android.files.util.UIUtil;
import com.ubuntuone.android.files.util.RegisterUserTask.RegisterUserTaskCallback;

public class SignUpFragment extends Fragment implements
		RegisterUserTaskCallback {
	private static final String TAG = SignUpFragment.class.getSimpleName();
	
	private SignUpFragmentCallback callback;

	private EditText fullnameEditText;
	private EditText usernameEditText;
	private EditText passwordEditText;

	private View captchaLoading;
	private Button captchaFailed;
	private View captchaReady;
	
	private AsyncImageView captchaImageView;
	
	private EditText captchaEditText;
	
	private Button signupButton;
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		callback = (SignUpFragmentCallback) getActivity();
	}

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container,
			Bundle savedInstanceState) {
		final View content = inflater.inflate(R.layout.fragment_sign_up,
				container, false);
		setupViews(content);	
		return content;
	}

	private void setupViews(View content) {
		fullnameEditText = (EditText) content.findViewById(R.id.sso_fullname);
		usernameEditText = (EditText) content.findViewById(R.id.sso_username);
		passwordEditText = (EditText) content.findViewById(R.id.sso_password);
		signupButton = (Button) content.findViewById(R.id.sso_signup);
		
		signupButton.setOnClickListener(onButtonClickedListener);

		final TextView footer = (TextView) content.findViewById(R.id.signup_footer_tos);
		footer.setText(Html.fromHtml(getString(R.string.signup_footer_tos)));
		footer.setMovementMethod(LinkMovementMethod.getInstance());
		
		setupCaptchaImageView(content);
	}
	
	private OnClickListener onButtonClickedListener = new OnClickListener() {
		@Override
		public void onClick(View v) {
			switch (v.getId()) {
			case R.id.sso_signup:
				onSignUpButtonClicked();
				break;
			}
		}
	};
	
	private void setupCaptchaImageView(View content) {
		captchaLoading = content.findViewById(R.id.sso_captcha_loading);
		captchaFailed = (Button) content.findViewById(R.id.sso_captcha_failed);
		captchaReady = content.findViewById(R.id.sso_captcha_ready);
		captchaReady.setVisibility(View.GONE);

		captchaImageView = (AsyncImageView) content.findViewById(R.id.sso_captcha_image);
		captchaEditText = (EditText) content.findViewById(R.id.sso_captcha);
		
		captchaImageView.setOnImageViewLoadListener(new OnImageViewLoadListener() {
			
			@Override
			public void onLoadingStarted(AsyncImageView imageView) {
				onCaptchaLoadingStarted();
			}
			
			@Override
			public void onLoadingFailed(AsyncImageView imageView, Throwable throwable) {
				onCaptchaLoadingFailed(throwable);
			}
			
			@Override
			public void onLoadingEnded(AsyncImageView imageView, Bitmap image) {
				onCaptchaLoadingEnded();
			}
		});
		
		captchaImageView.setClickable(true);
		captchaImageView.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				getNewCaptchaAsync();
			}
		});
		
		captchaFailed.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				onCaptchaLoadingStarted();
				getNewCaptchaAsync();
			}
		});
	}

	@Override
	public void onResume() {
		super.onResume();
		getNewCaptchaAsync();
	}
	
	public void getNewCaptchaAsync() {
		new GetNewCaptchaTask(
				new WeakReference<AsyncImageView>(captchaImageView)).execute();
	}

	private void onCaptchaLoadingStarted() {
		captchaLoading.setVisibility(View.VISIBLE);
		captchaFailed.setVisibility(View.GONE);
		// Don't hide captchaReady, captcha should be focused.
	}

	private void onCaptchaLoadingFailed(Throwable t) {
		captchaLoading.setVisibility(View.GONE);
		captchaFailed.setVisibility(View.VISIBLE);
		captchaReady.setVisibility(View.GONE);
		signupButton.setEnabled(false);
	}

	private void onCaptchaLoadingEnded() {
		captchaLoading.setVisibility(View.GONE);
		captchaFailed.setVisibility(View.GONE);
		captchaReady.setVisibility(View.VISIBLE);
		signupButton.setEnabled(true);
	}
	
	private class GetNewCaptchaTask extends AsyncTask<Void, Void, CaptchaResponse> {
		private WeakReference<AsyncImageView> viewRef;
		
		private String errorMessage;
		
		public GetNewCaptchaTask(WeakReference<AsyncImageView> view) {
			this.viewRef = view;
		}
		
		@Override
		protected void onPreExecute() {
			super.onPreExecute();
			if (captchaEditText != null)
				captchaEditText.setText("");
		}

		@Override
		protected CaptchaResponse doInBackground(Void... params) {
			final UbuntuSingleSignOnAPI api = new UbuntuSingleSignOnAPI(
					HttpManager.getClient(),
					Constants.SSO_SCHEME, Constants.SSO_HOST, null);
			
			try {
				return api.newCaptcha();
			} catch (Exception e) {
				return null;
			}
		}

		@Override
		protected void onPostExecute(CaptchaResponse result) {
			super.onPostExecute(result);
			final AsyncImageView view = this.viewRef.get();
			if (view != null && result != null) {
				captchaImageView.setTag(result.captchaId);
				captchaImageView.setUrl(result.imageUrl);
			} else {
				onCaptchaLoadingFailed(new Throwable(errorMessage));
			}
		}
	}
	
	private void onSignUpButtonClicked() {
		// Dismiss soft input method.
		getActivity().getWindow().setSoftInputMode(
				WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_HIDDEN);
		
		final String fullname = fullnameEditText.getText().toString();
		final String username = usernameEditText.getText().toString();
		final String password = passwordEditText.getText().toString();
		final String captchaId = (String) captchaImageView.getTag();
		final String captchaSolution = captchaEditText.getText().toString();
		
		registerUserAsync(fullname, username, password,
				captchaId, captchaSolution, this);
	}
	
	private void registerUserAsync(String fullname, String username,
			String password, String captchaId, String captchaSolution,
			RegisterUserTaskCallback callback) {
		new RegisterUserTask(getActivity(), callback)
				.execute(fullname, username, password,
							captchaId, captchaSolution);
	}

	@Override
	public void onRegisterUserSuccess(String oauthData) {
		Log.d(TAG, "onRegisterUserSuccess()");
		
		final Activity activity = getActivity();
		activity.setResult(Activity.RESULT_OK);
		activity.finish();
	}
	
	@Override
	public void onRegisterUserCancel() {
		Log.d(TAG, "onRegisterUserCancel()");
		
		UIUtil.showToast(getActivity(), "Registration canceled.", true);
		getNewCaptchaAsync();
	}
	
	@Override
	public void onRegisterUserAccountNotValidated(String oauthData) {
		Log.d(TAG, "onRegisterUserAccountNotValidatedException()");
		callback.requestedValidation();
	}

	@Override
	public void onRegisterUserAuthenticationException(Exception e) {
		Log.d(TAG, "onRegisterUserAuthenticationException()");
		
		UIUtil.showToast(getActivity(), "Authentication error: " + e.getMessage(), true);
		getNewCaptchaAsync();
	}

	@Override
	public void onRegisterUserRegistrationException(ArrayList<Error> errors) {
		Log.d(TAG, "onRegisterUserRegistrationException()");
		
		for (ServerResponse.Error error : errors) {
			String errorMessage = error.getDetails().get(0).toString();
			if ("email".equals(error.getReason())) {
				usernameEditText.setError(errorMessage);
			} else if ("password".equals(error.getReason())) {
				passwordEditText.setError(errorMessage);
			} else if ("captcha_solution".equals(error.getReason())) {
				captchaEditText.setError(errorMessage);
			} else if ("__all__".equals(error.getReason())) {
				UIUtil.showToast(getActivity(), errorMessage, true);
			}
		}
		getNewCaptchaAsync();
	}
	
	@Override
	public void onRegisterUserIOException(Exception e) {
		Log.d(TAG, "onRegisterUserIOException()");
		
		UIUtil.showToast(getActivity(), R.string.no_network, true);
		getNewCaptchaAsync();
	}
	
	@Override
	public void onRegisterUserGenericException(Exception e) {
		Log.d(TAG, "onRegisterUserGenericException()");
		
		UIUtil.showToast(getActivity(), "Registration error: " + e.getMessage(), true);
		getNewCaptchaAsync();
	}
	
	public interface SignUpFragmentCallback {
		public void requestedValidation();
	}
}
