/*
 * Ubuntu One Files - access Ubuntu One cloud storage on Android platform.
 * 
 * Copyright (C) 2011 Canonical Ltd.
 * Author: Michał Karnicki <michal.karnicki@canonical.com>
 *   
 * This file is part of Ubuntu One Files.
 *  
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *  
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.files.util;

import java.util.List;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.TextView;

import com.ubuntuone.android.files.R;
import com.ubuntuone.android.files.util.ServicesStatus.ServiceStatusRecord;
import com.ubuntuone.android.files.util.ServicesStatus.StatusCodes;

public class ServicesAdapter extends ArrayAdapter<ServicesStatus.ServiceStatusRecord>{
	
	private Context context;
	private List<ServicesStatus.ServiceStatusRecord> services;

	public ServicesAdapter(Context context, int textViewResourceId,
			List<ServiceStatusRecord> objects) {
		super(context, textViewResourceId, objects);
		this.context = context;
		this.services = objects;
	}

	@Override
	public View getView(int position, View convertView, ViewGroup parent) {
		// We don't use convertView, list is short. Plus, it messed with
		// the list separator.
		View view;
		ServiceStatusRecord record = services.get(position);
		
		LayoutInflater layoutInflater = 
			(LayoutInflater)context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		if (record == null) {
			view = layoutInflater.inflate(R.layout.list_status_row, null);
			return view;
		}
		
		// TODO Make description items not focusable/clicable
		if (record.status == 0) {
			// separator
			view = layoutInflater.inflate(R.layout.list_separator, null);
			view.setFocusable(false);
			view.setClickable(false);
		} else {
			view = layoutInflater.inflate(R.layout.list_status_row, null);
			if (record.status < 0) {
				// bottom descriptive items
				view.setFocusable(false);
				view.setClickable(false);
				record.status *= -1;
			} else {
				// regular service
				view.setFocusable(true);
				view.setClickable(true);
			}
		}
	
		if (record != null) {
			if (record.status != 0) {
				// service name
				TextView textView = (TextView)view.findViewById(R.id.status_text);
				if (textView != null)
					textView.setText(record.service);
				// service status image
				ImageView imageView = (ImageView)view.findViewById(R.id.status_image);
				if (imageView != null) {
					imageView.setVisibility(View.VISIBLE);
					imageView.setImageDrawable(ServicesStatus.getStatusDrawable(context, record.status));
				}
				// background
				
				switch (record.status) {
				case StatusCodes.STATUS_OK:
					view.setBackgroundDrawable(context.getResources()
							.getDrawable(R.drawable.status_bg_ok));
					break;
				case StatusCodes.STATUS_DISRUPTION:
					view.setBackgroundDrawable(context.getResources()
							.getDrawable(R.drawable.status_bg_disruption));
					break;
				case StatusCodes.STATUS_OUTAGE:
					view.setBackgroundDrawable(context.getResources()
							.getDrawable(R.drawable.status_bg_outage));
					break;
				case StatusCodes.STATUS_INFO:
					view.setBackgroundDrawable(context.getResources()
							.getDrawable(R.drawable.status_bg_info));
					break;
				}
			}
		}
		return view;
	}
	
}
